# README for "Increasing surface runoff from Greenland's firn areas"

This repository contains the scripts used for all the analysis of this paper.

The processing workflow which was used to create the visible runoff limits dataset is found in the `river_detection` repository.

See also the associated data repository of this paper.


## Basics

* Most scripts in this repository rely on `load_env.py` to import common packages, plotting styles, datasets, region definitions, common paths, and also to set up cluster compute when needed. 
* Several of the scripts originate as, or are paired with Jupyter notebooks, using Jupytext functionality. They are identified with the :notebook: icon.
* Take note also of system-specific paths defined as environment variables. On the `beo05` system these are found in `~/source_rivers.sh`.


## GIS datasets

For more information on the provenance of these datasets, see the README of the data repository.

* GIMPDEM 
* Greenland Ice Sheet outline 
* 'No-zone' areas (created manually in QGIS)
* PostgreSQL database - generate your own from the csvs in all_filtered_csv/.
* `boxes_high_points.shp` - needed to produce legal polygons of `e_max` metrics.



## Data reduction

* Data reduction via 100 km boxes:
    - First generate the 100 km boxes: `make_100k_bins.py`, with manual editing undertaken after in QGIS, including adding a field manually specifying draw direction.
        + Creates `boxes.shp`, which cannot be automatically regenerated.
    - `make_sub_bins.py` to create the 1 km slices which divide up each 100 km box.
        + Creates `boxes_fid{b}.shp`. These can be automatically regenerated provided that `boxes.shp` is available as input.
    - Then, `find_annual_rlims.py` uses `rlims` PostgreSQL database to generate:
        + `100km_box_medians.csv`
        + `xytpd.csv` : annual maximum runoff limit location every 1 km for whole GrIS.
        + This script is where the clustering approach to calculate median absolute deviations is executed. Initial investigations of this approach were made in `investigate_clustering_for_uncertainty.ipynb` :notebook:

* The polygons of GrIS-wide annual and multi-annual runoff limits are generated with `build_rlim_polygons.py` using data from `xytpd.csv`.
    - Some more verbose testing functionality can still be seen in 
    `test_contour_generation.py` :notebook:
    - With the `-restrict` option, the output polygons show the runoff limit only in regions listed as valid by `load_env.py --> regions_boxes`. Runoff limits outside these areas are set to the ice sheet margins.

* Building 'uncertainty' polygons - +/- 1 MAD, and estimate of maximum annual likely runoff limit:
    1. Requires `slice_centreline_elevations.py` to be run first. Extracts from GIMPDEM into CSV files for each slice.
    2. MAD values are provided in `xytpd.csv` output by `find_annual_rlims.py`.
    3. Run `seasonality_for_max_rlim.py` to calculate estimates of maximum annual runoff limit elevation. This outputs `annual_emax.csv`.
    4. Run `estimate_coords_from_elev.py`. This generates alternative versions of `xytpd.csv` for -MAD, +MAD and annual maximums.
    5. Run `build_rlim_polygons.py` to build the polygons, using the `-xyt` argument, e.g. `python build_rlim_polygons.py 1985 2020 -xyt xytpd_region_emax.csv -restrict`. Add `-interpolate` flag if needed (this is optional as of 2021-09-22).
    6. For use with the RCM outputs, these polygons need to be rasterised. Use  `weight_rcm_runoff.py` for this - the name of the script is not that self-explanatory! But this script can rasterize for either of MAR or RACMO versions specified in load_env.py, has a choice between binary or fractional ('weighted') masking, and can produce masks for any of the output polygon types (e.g. `xytpd_minus_mad`).


### Manual editing of 1985-92 and 2013-20 limits

Manual/visual delineation of tricky parts of the NE and NW, for the two 'common' periods only. This must be applied post-hoc to the automatically-generated common period runoff limit polygons.

* `rlims-landsat-mosaics.js` - a Javascript file, run on Google Earth Engine, to produce multi-year Landsat mosaics for the NE and NW sectors of interest
* `rlims-landsat-mosaics-use-assets.js` - Use on Google Earth Engine. Plots Landsat mosaics for user, enabling them to digitise the areas to change; polygon definitions are then stored as inputs in this same JS file and are saved as shapefiles to Google Drive.
* `manual_rlim_edits.py` - uses shapefiles from GDrive, substitutes them into the common period shapefiles.


## RCM data

## Download RACMO data

It's over HTTP, so recursive SCP does not work. Use wget instead:

    wget -r -np <link>


## RCM comparisons

* `calculate_mar_excess_melt.py`
* `calculate_racmo_excess_melt.py`
* `regions_to_rcm_masks.py` - this creates rasters of each defined region (e.g. S, SSW...) for each of MAR and RACMO.
* `weight_rcm_runoff.py` - creates rasters of each annual/multi-annual runoff limit polygon
    - For MAR with weights for the cells that are crossed by the runoff limit
    - For RACMO@1km, simple binary polygons.
    - Outputs `.nc` files which are used in statistics/figure production below.



## Figure-by-Figure explanation of scripts

### Figure 1: Runoff limit elevation

* Map: `plot_map_decadal_change.py` :notebook:
    - uses multi-annual polygons.
    - Also plots a manually-created shapefile of region divides previously created in QGIS.
    - Uses the include file `scalebar.py` included in this repo.
* Heatmaps: `plot_rlim_heatmap.py` :notebook: 
* Region timeseries and trends: `plot_rlim_heatmap.py` :notebook:
* Then composed in Illustrator.


### Figure 2: Relationship between excess melt and runoff limit

* `calculate_mar_excess_melt.py` :notebook:
* `plot_rlim_vs_excess_melt.py` :notebook:


### Figure 2: Changes in runoff area and quantity

* `plot_runoff.py` :notebook:


### EDFig 1: Methods - image processing

* `plot_methods.py` :notebook:


### EDFig 2: Methods - example retrievals

* `plot_methods.py` :notebook:


### EDFig 3: Methods - aggregation

* `plot_methods.py` :notebook:
* Then Illustrator: `fig_methods_composite.ai`


### EDFig 4: Impact of spatial (elevation) coverage on captured runoff limit

* `check_spatial_cover_1km.py` :notebook:

N.b. this script requires that counts have been generated according to the instructions in `river_detection/README` - in summary, use `create_master_maps.py` to produce
 `pixel_counts/master_{y}_count_matchgimp_1km.vrt` with underlying tiff tiles, per year.


### EDFig 5: Influence of satellite acquisition date on runoff limit

In submission 2, this is implemented via the 'estimated maximum runoff limit' approach, using seasonality. Plotting:

* `suppl_runoff_limit_emax.py` :notebook:


### EDFig 6: Break-year statistics

* `plot_rlim_vs_excess_melt.py` :notebook:


### EDFig 7: Coincidence with ice slabs

* `plot_map_decadal_change.py` :notebook: 


### EDFig 8: RCM runoff areas

* `plot_runoff.py`(section 'Sensitivity analysis of threshold choice on area')


### EDFig 9: Regional estimates of modelled runoff area and volume

* `plot_runoff.py`















