"""

Given an elevation in a box-slice combination, return the likely x/y coordinate
position.

Pre-requisites:
- Per-slice DEM csvs created by slice_centreline_elevations.py
- xytpd (via load_env.py, find_annual_rlims.py)
- annual_emax.csv generated by seasonality_for_max_rlim.py

This has not yet been parallelised, it would be a good candidate on a per-box basis though
as it takes 10s of minutes to run.

AT July/August 2021
See also investigate_clustering_for_uncertainty.ipynb.

"""

import pandas as pd
import geopandas as gpd
from shapely.geometry import Point
from scipy.spatial import cKDTree

from load_env import * 
gis = gisloader()
rlims = rlimsloader()

# ------------------------------------------------------------------------------

run_minus_mad = False
run_plus_mad = False
run_emax = True

# ------------------------------------------------------------------------------

annual_emax = pd.read_csv(os.path.join(os.environ['ANALYSIS_OUT'], 
    'seasonality', 'annual_emax_region_poly2.csv'))
annual_emax = annual_emax.filter(items=['year','box_id','slice_id','est_max'], 
    axis='columns')
annual_emax = pd.merge(rlims.xytpd, annual_emax,
    on=['year','box_id','slice_id'])
annual_emax = annual_emax.rename({'est_max':'nz', 'x':'ox', 'y':'oy'},
    axis='columns')

# ------------------------------------------------------------------------------




xyzt_minus_mad = []
xyzt_plus_mad = []
xyzt_emax = []

#limited = gis.boxes.iloc[14:16]
#for ix, box in limited.iterrows():
for ix, box in gis.boxes.iterrows():

    print(ix)

    slices = gpd.read_file('/home/geoscience/nobackup_cassandra/L0data/100k_boxes/boxes_fid{:d}.shp'.format(box.FID))
    for s_ix, s in slices.iterrows():
        
        sl_dem = pd.read_csv(os.path.join(os.environ['ANALYSIS_OUT'],
                    'slice_centre_elevations',
                    'slice_elevations_box{n}_s{s}.csv'.format(n=box.FID, s=s_ix)))
        sl_dem = gpd.GeoDataFrame(sl_dem, 
            geometry=gpd.points_from_xy(sl_dem.x, sl_dem.y))
        
        def _finder(ee):
            """ 
            [DEFUNCT]
            Find the coordinates of the requested elevation by simply
            returning the first row which is >= the elevation.
            """
            res = sl_dem[sl_dem.z >= ee]
            if len(res) > 0:
                return res.iloc[0]
            else:
                res = sl_dem.iloc[-1]
                res['x'] = 0
                res['y'] = 0
                return res 

        def _gpd_finder(ee, op):
            """
            Uses the point of origin and the "new" elevation to find
            the horizontally-closest DEM coordinate which satisfies
            the new elevation.
            """
            if op == 'gt':
                pts = sl_dem[(sl_dem.z >= ee.nz)]
            elif op == 'lt':
                pts = sl_dem[(sl_dem.z <= ee.nz)]
            else:
                raise ValueError
            
            if len(pts) > 0:
                pts.loc[:,'d'] = pts.distance(Point(ee.ox, ee.oy))
                return pts.sort_values('d').iloc[0]
            else:
                res = sl_dem.iloc[-1]
                res['x'] = 0
                res['y'] = 0
                return res

        def _add_labels(df, src):
            """
            Adds labels which are needed for later parts of the workflow.
            """
            to_add = src.filter(items=['year', 'loc_id', 'slice_id', 'box_id'], axis='columns')
            return pd.merge(df, to_add, left_index=True, right_index=True)


        ## +/- 1 MAD - calculate the elevations from xytpd:
        pts_in_slice = rlims.xytpd[(rlims.xytpd.box_id == box.FID) & (rlims.xytpd.slice_id == s_ix)]

        if len(pts_in_slice) > 0:
            if run_minus_mad:
                e_minus_mad = pts_in_slice.elev - pts_in_slice.cluster_mad
                e_minus_mad.name = 'nz'
                e_minus_mad = e_minus_mad.to_frame()
                e_minus_mad.loc[:,'ox'] = pts_in_slice.x
                e_minus_mad.loc[:,'oy'] = pts_in_slice.y
                xyz_minus_mad = e_minus_mad.apply(_gpd_finder, op='lt', axis='columns') 
                xyz_minus_mad = _add_labels(xyz_minus_mad, pts_in_slice)
                xyzt_minus_mad.append(xyz_minus_mad)
            
            if run_plus_mad:
                e_plus_mad = pts_in_slice.elev + pts_in_slice.cluster_mad
                e_plus_mad.name = 'nz'
                e_plus_mad = e_plus_mad.to_frame()
                e_plus_mad.loc[:,'ox'] = pts_in_slice.x
                e_plus_mad.loc[:,'oy'] = pts_in_slice.y
                xyz_plus_mad = e_plus_mad.apply(_gpd_finder, op='gt', axis='columns') 
                xyz_plus_mad = _add_labels(xyz_plus_mad, pts_in_slice)
                xyzt_plus_mad.append(xyz_plus_mad)

        ## Estimated (Emax) from seasonal polynomials
        # Need to add the original x,y coordinates back onto this.
        if run_emax:
            slice_emax = annual_emax[(annual_emax.box_id == box.FID) & (annual_emax.slice_id == s_ix)]
            if len(slice_emax) > 0:
                xyz_slice_emax = slice_emax.apply(_gpd_finder, op='gt', axis='columns')
                xyz_slice_emax = _add_labels(xyz_slice_emax, slice_emax)
                xyzt_emax.append(xyz_slice_emax)

        sl_dem = None

if run_minus_mad:
    xyzt_minus_mad = pd.concat(xyzt_minus_mad, axis=0)
    xyzt_minus_mad.to_csv(os.path.join(os.environ['ANALYSIS_OUT'],
        'rlim_annual_maxm',
        'xytpd_minus_mad.csv'))

if run_plus_mad:    
    xyzt_plus_mad = pd.concat(xyzt_plus_mad, axis=0)
    xyzt_plus_mad.to_csv(os.path.join(os.environ['ANALYSIS_OUT'],
        'rlim_annual_maxm',
        'xytpd_plus_mad.csv'))

if run_emax:
    xyzt_emax = pd.concat(xyzt_emax, axis=0)
    xyzt_emax.to_csv(os.path.join(os.environ['ANALYSIS_OUT'],
        'rlim_annual_maxm',
        'xytpd_region_emax.csv'))
