# %% trusted=true
import matplotlib.pyplot as plt
import os
import geoutils as gu
import numpy as np
import pandas as pd
from matplotlib import colors
import geopandas as gpd

# %% trusted=true
import seaborn as sns


# %% trusted=true
from load_env import *

# %% trusted=true
sns.set_context('paper', rc={'font.family':'DIN Alternate', 'font.size':9})
sns.set_context('paper', rc={'font.family':'Arial'})
fs = (6,3.5)
sns.set_style('ticks')
# %matplotlib widget

# %% trusted=true
#pth = '/scratch/LC8_example.20210602-095748.job/LC08_L1TP_006013_20190813_20190820_01_T1'
pth = '/flash/tedstona/LC8_example.20210602-095748.job/LC08_L1TP_006013_20190813_20190820_01_T1'
ds = 4
region = [-115855.489,-2444552.170,-78927.297,-2487548.980]
# floats (left, right, bottom, top)
extent = [region[0], region[2], region[3], region[1]]

# %% trusted=true
b5 = gu.Raster(os.path.join(pth, 'LC08_L1TP_006013_20190813_20190820_01_T1_B5_EPSG3413.tif'),
                load_data=False)
b5.crop(region)
b5.load()

# %% trusted=true
s1_med = gu.Raster(os.path.join(pth, 'LC08_L1TP_006013_20190813_20190820_01_T1_S1_filt_med_EPSG3413.tif'),
                load_data=False)
s1_med.crop(region)
s1_med.load()

# %% trusted=true
s2_hipass = gu.Raster(os.path.join(pth, 'LC08_L1TP_006013_20190813_20190820_01_T1_S2_hipass_EPSG3413.tif'),
                load_data=False)
s2_hipass.crop(region)
s2_hipass.load()

# %% trusted=true
s3_obnlm = gu.Raster(os.path.join(pth, 'LC08_L1TP_006013_20190813_20190820_01_T1_S3_obnlm_EPSG3413.tif'),
                load_data=False)
s3_obnlm.crop(region)
s3_obnlm.load()

# %% trusted=true
s4_gabor = gu.Raster(os.path.join(pth, 'LC08_L1TP_006013_20190813_20190820_01_T1_S4_gabor_EPSG3413.tif'),
                load_data=False)
s4_gabor.crop(region)
s4_gabor.load()

# %% trusted=true
s5_opening = gu.Raster(os.path.join(pth, 'LC08_L1TP_006013_20190813_20190820_01_T1_S5_opening_EPSG3413.tif'),
                load_data=False)
s5_opening.crop(region)
s5_opening.load()

# %% trusted=true
labels = gu.Raster(os.path.join(pth, 'LC08_L1TP_006013_20190813_20190820_01_T1_EPSG3413_LABELS.tif'),
                load_data=False)
labels.crop(region)
labels.load()

# %% trusted=true
filtlabels = gu.Raster(os.path.join(pth, 'LC08_L1TP_006013_20190813_20190820_01_T1_LABELS_FILT.tif'),
                load_data=False)
filtlabels.crop(region)
filtlabels.load()


# %% trusted=true
counts, values = np.histogram(labels.data[0,:,:], bins=300, range=(0,300))
fcounts, fvalues = np.histogram(filtlabels.data[0,:,:], bins=300, range=(0,300))
n = 0
for c, v, fc, fv in zip(counts.flatten(), values.flatten(), fcounts.flatten(), fvalues.flatten()):
    if c > 0:
        labels.data = np.where(labels.data == v, n, labels.data)
        
        if fc > 0:
            filtlabels.data = np.where(filtlabels.data == fv, n, filtlabels.data)

        n += 1


# %% trusted=true
plt.figure(figsize=(fs))

# %% trusted=true
axes = []

# %% trusted=true
ax = plt.subplot(2,4,1)
plt.imshow(s1_med.data[0,:,:], cmap='Greys_r', interpolation='none', extent=extent, zorder=1)

# %% trusted=true
xd = (region[2] - region[0]) / 6
yp = (region[3] - region[1]) / 8
xx = [(region[0] + (xd*3)), (region[0] + (xd*5))]
yy = [region[1]+yp, region[1]+yp]
plt.plot(xx, yy, linewidth=2, color='black', zorder=10)
print(xd)

# %% trusted=true
#plt.xlim(region[0], region[2])
#plt.ylim(region[1], region[3])
plt.title('1. Original data')
axes.append(ax)

# %% trusted=true
ax = plt.subplot(2,4,2)
plt.imshow(s1_med.data[0,:,:], cmap='Greys_r', interpolation='none')
plt.title('2. Median Filter')
axes.append(ax)

# %% trusted=true
ax = plt.subplot(2,4,3)
plt.imshow(s2_hipass.data[0,:,:], interpolation='none', vmin=200, vmax=800)
plt.title('3. FFT High-Pass')
axes.append(ax)

# %% trusted=true
ax = plt.subplot(2,4,4)
plt.imshow(s3_obnlm.data[0,:,:], interpolation='none', vmin=200, vmax=800)
plt.title('4. OBNLM Filter')
axes.append(ax)

# %% trusted=true
ax = plt.subplot(2,4,5)
plt.imshow(s4_gabor.data[0,:,:], interpolation='none', vmin=40, vmax=200)
plt.title('5. Gabor Filter')
axes.append(ax)

# %% trusted=true
ax = plt.subplot(2,4,6)
plt.imshow(s5_opening.data[0,:,:], interpolation='none', vmin=40, vmax=200)
plt.title('6. Path Opening')
axes.append(ax)

# %% trusted=true
# make a color map of fixed colors
cmap = colors.ListedColormap(sns.color_palette('muted').as_hex()[:7])
bounds = np.arange(0,7,1)
norm = colors.BoundaryNorm(bounds, cmap.N)

# %% trusted=true
ax = plt.subplot(2,4,7)
plt.imshow(labels.data[0,:,:], interpolation='none', cmap=cmap, norm=norm)
plt.title('7. Segment')
axes.append(ax)

# %% trusted=true
ax = plt.subplot(2,4,8)
plt.imshow(filtlabels.data[0,:,:], interpolation='none', cmap=cmap, norm=norm)
plt.title('8. Filter Labels')
axes.append(ax)

# %% trusted=true
for ax in axes:
    ax.get_xaxis().set_visible(False)
    ax.get_yaxis().set_visible(False)
    #plt.yticks([])
    #plt.xticks([])


# %% trusted=true
plt.tight_layout()


# %% trusted=true
plt.savefig(os.path.join(os.environ['FIGS_OUT'], 'methods_extraction_overview.pdf'), dpi=300)

# %% [markdown]
# ===========================

# %% trusted=true
#rlims = pd.read_csv(os.path.join(pth, 'LC08_L1TP_006013_20190813_20190820_01_T1_RLIM.csv'))
filt_rlims = pd.read_csv('/flash/tedstona/LC8_example.20210602-095748.job/LC8_example.20210602-095748.job_RLIM_filtered_MM.csv')
b5_all = gu.Raster(os.path.join(pth, 'LC08_L1TP_006013_20190813_20190820_01_T1_B5_EPSG3413.tif'))
limit = pd.read_csv(os.path.join(os.environ['ANALYSIS_OUT'], 'runoff_limit_polys/poly_2019.csv'))
poly19 = gpd.read_file(os.path.join(os.environ['ANALYSIS_OUT'], 'runoff_limit_polys/poly_2019.shp'))
poly19_lo = gpd.read_file(os.path.join(os.environ['ANALYSIS_OUT'], 'runoff_limit_polys/poly_2019_xytpd_minus_mad.shp'))
poly19_hi = gpd.read_file(os.path.join(os.environ['ANALYSIS_OUT'], 'runoff_limit_polys/poly_2019_xytpd_plus_mad.shp'))

# %% trusted=true
# %matplotlib widget
plt.figure(figsize=(3,3))
ax = plt.subplot(111)

#plt.imshow(filtlabels.data[0,:,:], interpolation='none', cmap=cmap, norm=norm, extent=extent, alpha=1)
plt.imshow(s1_med.data[0,:,:], cmap='Greys_r', interpolation='none', extent=extent, alpha=0.8)
#b5_all.show(ax=ax, cmap='Greys_r', add_cb=False)
##plt.imshow(b5_all.data[0,:,:], interpolation='none', extent=extent, origin='lower', cmap='Greys_r')
#plt.plot(rlims.x, rlims.y, 'o', mec='tab:red', mfc='none', markersize=2, alpha=0.6, zorder=1)

#c_thresh = 500
#ff = rlims[((rlims.px_to_cld_up > c_thresh) | (rlims.px_to_cld_up < 2)) &
#                  ((rlims.px_to_cld_down > c_thresh) | (rlims.px_to_cld_down < 1))]
#plt.plot(ff.x, ff.y, 'o', mec='tab:purple', mfc='tab:purple', markersize=3, zorder=10)

plt.plot(filt_rlims.x, filt_rlims.y, 'o', mec='#ffc400', mfc='none', markersize=1, alpha=1)

orange = '#ff7c00'
plt.plot(limit['x'], limit['y'], alpha=1, color=orange, marker='o', mfc='none', markersize=2, linestyle='none')

# To save clipping hassle in Illustrator later, pre-clip the elevation lines.
# This means actually first converting them to linestrings, then clipping'
# If they remained as polygons then intersect would insist on closing the polygon.
from shapely.geometry import box
bbox = box(*region)
poly19.geometry.boundary.intersection(bbox).plot(ax=ax, color=orange, zorder=1000, linewidth=2, linestyle='--')
poly19_lo.geometry.boundary.intersection(bbox).plot(ax=ax, color=orange, zorder=1000, linewidth=1, linestyle=':')
poly19_hi.geometry.boundary.intersection(bbox).plot(ax=ax, color=orange, zorder=1000, linewidth=1, linestyle=':')


#plt.plot(limit['xs'], limit['ys'], alpha=0.7, color='purple')
plt.xlim(region[0], region[2])
plt.ylim(region[3], region[1])

ax.get_xaxis().set_visible(False)
ax.get_yaxis().set_visible(False)

plt.savefig(os.path.join(os.environ['FIGS_OUT'], 
                         'methods_scene_runoff_limit.pdf'), dpi=300, bbox_inches='tight')
plt.savefig(os.path.join(os.environ['FIGS_OUT'], 
                         'methods_scene_runoff_limit.tif'), dpi=300, bbox_inches='tight')

# %% trusted=true
plt.figure()
poly19.geometry.boundary.intersection(bbox).plot()

# %% [markdown]
# HAving problems getting the x/y positions to draw on top of the image. plotting them on top of b5 
# image in QGIS works fine though


# %% [markdown]
# Then a final B5 with the "raw" runoff limit points on it, and the final ones presented in the paper

# %% trusted=true
lims_im = limit[(limit.box_id == 7) & (limit.date == '2019-08-13')]
plt.figure()
ax = plt.subplot(111)
#plt.imshow(filtlabels.data[0,:,:], interpolation='none', cmap=cmap, norm=norm, extent=extent)
b5_all.show(ax=ax)
plt.plot(lims_im.x, lims_im.y, 'x')

# %% [markdown]
# ## 'Raw' images and filtered runoff limits

# %% trusted=true
from glob import glob
jobs = glob(os.path.join(os.environ['FINAL_OUTPUTS'], '*.job'))

    # %% trusted=true
    for j in jobs:
        product = os.path.join(*images[0].split('/')[1:-1])
        pth = os.path.join(j, product)
        r = os.path.exists(pth)
        print(pth)
        if r:
            print('\t {j}'.format(j=j))
            break

# %% trusted=true
image.filename.split('/')[-2:-1]

# %% trusted=true
# %matplotlib widget
plt.figure(figsize=(6,8))
nr = 3
nc = 2

letgen = letters()
from more_itertools import peekable
letgen = peekable(letgen)
from shapely.geometry import box

def _format_common(ax, label=True):
    sns.despine(ax=ax, top=True, bottom=True, left=True, right=True)
    ax.set_yticks([])
    ax.set_xticks([])
    if label:
        ax.annotate(next(letgen), fontweight='bold', xy=(0.05, 0.95), xycoords='axes fraction',
           horizontalalignment='left', verticalalignment='top', color='w')
    
plt_kws = dict(cmap='Greys_r', add_cb=False)
plt_kws = dict(cmap='Greys_r')

images = [
    #'TM/LT05_L1TP_014002_19950803_20180220_01_T1/LT05_L1TP_014002_19950803_20180220_01_T1_B4.TIF',
    'TM/LT05_L1TP_020001_19950712_20180221_01_T1/LT05_L1TP_020001_19950712_20180221_01_T1_B4.TIF',
    'TM/LT05_L1TP_035003_19950705_20180221_01_T1/LT05_L1TP_035003_19950705_20180221_01_T1_B4.TIF',
    'TM/LT05_L1GS_021006_19970809_20180221_01_T2/LT05_L1GS_021006_19970809_20180221_01_T2_B4.TIF',
    'ETM/LE07_L1TP_011010_20090812_20161218_01_T1/LE07_L1TP_011010_20090812_20161218_01_T1_B4.TIF',
    'OLI_TIRS/LC08_L1TP_006013_20190813_20190820_01_T1/LC08_L1TP_006013_20190813_20190820_01_T1_B5.TIF',
    'ETM/LE07_L1TP_005015_20110808_20161207_01_T1/LE07_L1TP_005015_20110808_20161207_01_T1_B4.TIF'    
]

from glob import glob
jobs = glob(os.path.join(os.environ['FINAL_OUTPUTS'], '*.job'))

pn = 1
bounds = {}
for im in images:
    
    # Prepare image and bounds
    image = gu.Raster(os.path.join(os.environ['L0lib'], im), downsample=4)
    geom = box(*image.get_bounds_projected(3413))
    label = letgen.peek()
    bounds[image.filename] = dict(geometry=geom, label=label)
    
    # Find the underlying job which processed the image
    found = False
    for j in jobs:
        product = os.path.join(*image.filename.split('/')[-2:-1])
        pth = os.path.join(j, product)
        r = os.path.exists(pth)
        if r:
            found = True
            break
    if not found:
        raise IOError()
        
    print('({l}) {p}'.format(l=label, p=product))
    
    # Get the semi-filtered points which were retained from the individual image
    rlims = pd.read_csv(os.path.join(j, j.split('/')[-1] + '_RLIM_filtered_MM.csv'))
    filt_rlims = rlims[rlims.PRODUCT_ID == product]
    filt_rlims = filt_rlims[filt_rlims.rlim == True]
    filt_rlims = gpd.GeoDataFrame(filt_rlims, geometry=gpd.points_from_xy(filt_rlims.x, filt_rlims.y), crs=3413)
    filt_rlims = filt_rlims.to_crs(image.crs)
    
    # Plot it
    ax = plt.subplot(nr, nc, pn)
    bb = image.bounds
    plt.imshow(image.data[0,:,:], extent=(bb.left, bb.right, bb.bottom, bb.top), **plt_kws)
    plt.plot(filt_rlims.geometry.x, filt_rlims.geometry.y, '.', color='tab:red', alpha=0.2, markersize=2)
    
    # Satellite and date
    p_split = product.split('_')
    sat_n = p_split[0][-1]
    if sat_n == '5':
        sensor = 'TM'
    elif sat_n == '7':
        sensor = 'ETM'
    elif sat_n == '8':
        sensor = 'OLI'
    
    sat_name = 'Landsat {i} ({s})'.format(i=sat_n, s=sensor)
    date = dt.datetime.strptime(p_split[3], '%Y%m%d').strftime('%Y-%m-%d')
    ax.annotate(sat_name + '\n' + date, xy=(0.05, 0.1), xycoords='axes fraction',
           horizontalalignment='left', verticalalignment='top', color='w', fontsize=6)
    
    _format_common(ax)
    
    # Increment subplot counter
    pn += 1
    

### Context map
# World land areas
greenland = gpd.read_file(os.path.join(os.environ['L0lib'], 'NaturalEarth/ne_10m_land/ne_10m_land.shp'))
# Crop world land areas to Greenland and surrounding areas
bbox = gpd.read_file(os.path.join(os.environ['L0lib'], 'greenland_region_bbox/greenland_area_bbox.shp')).to_crs(3413)
just_greenland = gpd.clip(greenland.to_crs(3413), bbox)
# Manually isolate contiguous Greenland polygon from the main multi-polygon.
jg = just_greenland.filter(items=[0], axis=0)
jgg = jg.loc[0].geometry
jgg_poly = jgg[9]
jgg_gdf = gpd.GeoDataFrame({'ix':[1,]}, geometry=[jgg_poly], crs=3413)

# left, bottom, width, height
ax_context = plt.axes((0.72, 0.6, 0.25, 0.25))
from shapely.geometry import box
gdf = gpd.GeoDataFrame(pd.DataFrame.from_dict(bounds, orient='index'), crs=3413, geometry='geometry')
jgg_gdf.plot(ax=ax_context, color='#ccc')
gdf.plot(ax=ax_context, edgecolor='black', color='none')
for ix, im in gdf.iterrows():
    xy = im.geometry.centroid.xy
    ax_context.annotate(im.label, xy=(xy[0][0], xy[1][0]), xycoords='data',
           horizontalalignment='center', verticalalignment='center')
ax_context.set_yticks([])
ax_context.set_xticks([])

plt.subplots_adjust(wspace=0.1, hspace=0.1)
plt.tight_layout()
plt.savefig(os.path.join(os.environ['FIGS_OUT'], 'suppl', 'methods_example_retrievals.pdf'), dpi=300)


# %% trusted=true
filt_rlims

# %% trusted=true
gdf

# %% trusted=true
import more_itertools

# %% trusted=true
p = more_itertools.peekable(letgen)

# %% trusted=true
next(letgen)

# %% trusted=true
p.peek()

# %% [markdown]
# ## Methods - data reduction

# %% trusted=true
from find_annual_rlims import *
engine = create_engine(db_href)
df = query_box(8, engine)
df = df[df.year == 2019]
slices_b8 = gpd.read_file('/home/geoscience/nobackup_cassandra/L0data/100k_boxes/boxes_fid{:d}.shp'.format(8))

from load_env import *

# %% trusted=true
rlims = rlimsloader()
gis = gisloader()

# %% trusted=true
labels = do_clustering(df)

# %% trusted=true
labels

# %% trusted=true
df_labelled

# %% trusted=true
import scalebar
import cartopy.crs as ccrs


# %% trusted=true
# %matplotlib widget

plt.figure(figsize=(4,3))

# Define the CartoPy CRS object.
crs = ccrs.NorthPolarStereo(central_longitude=-45., true_scale_latitude=70.)
ax = plt.axes(projection=crs)

# This can be converted into a `proj4` string/dict compatible with GeoPandas
crs_proj4 = crs.proj4_init

ax.set_facecolor('#ececec')


b = gis.boxes[gis.boxes.FID == 8].iloc[0]
bounds = slices_b8.total_bounds
bounds[2] = bounds[2] = 3e4
bbox = box(*bounds)

gis.gris_outline.intersection(bbox).plot(ax=ax, color='white', zorder=1)

plt.plot(*b.geometry.exterior.xy, zorder=1, color='grey', linestyle=':', alpha=0.3)

df_labelled = pd.merge(df, labels, 
        left_index=True, right_index=True, how='inner')
palette = sns.color_palette('muted', n_colors=len(df_labelled.label.unique()))
sns.scatterplot(data=df_labelled, x='x', y='y', hue='label', palette=palette,
               legend=False, alpha=0.4)

slices_b8_centre = slices_b8.loc[50]
plt.plot(*slices_b8_centre.geometry.exterior.xy, zorder=2, color='black', alpha=0.2)

import matplotlib.patches as mpatches
xsize = -85414 - -117842
ysize = -2553321 - -2543057
rect = mpatches.Rectangle((-117842, -2543057), xsize, ysize, linewidth=1, edgecolor='k', facecolor='none', linestyle='--')
# Add the patch to the Axes
ax.add_patch(rect)


ax.set_extent([bounds[0], bounds[2]-3e4, bounds[1], bounds[3]], crs=crs)
ax.gridlines(draw_labels=True, xlocs=[-50, -48], ylocs=[66.5, 67], x_inline=False, y_inline=False, zorder=120)

scalebar.scale_bar(ax, (0.7, 0.9), 30)


plt.savefig(os.path.join(os.environ['FIGS_OUT'], 
                         'methods_box8_2019_clusters_map.pdf'), dpi=300)

# %% trusted=true
# %matplotlib widget
plt.figure(figsize=(3,2))
ax = plt.subplot(111)

from matplotlib import dates
df_labelled.loc[:,'date'] = pd.to_datetime(df_labelled.date)
#sns.scatterplot(data=df_labelled, x='date', y='elev', hue='label', palette=palette, marker='o', alpha=0.1)
sns.histplot(data=df_labelled, x='date', y='elev', hue='label', palette=palette,
            legend=False)

val = rlims.xytpd[(rlims.xytpd.box_id == 8) & (rlims.xytpd.year == 2019) & (rlims.xytpd.slice_id==50)]
val = val.iloc[0]
plt.plot(val.date, val.elev, 'o', mec='black', mfc='none', markersize=6)

ax.xaxis.set_major_locator(dates.WeekdayLocator(interval=2))
ax.xaxis.set_major_formatter(dates.DateFormatter('%d/%m'))
sns.despine()
plt.ylabel(r'$\grave \upsilon$ (m)')
plt.xlabel('2019')
plt.tight_layout()

plt.savefig(os.path.join(os.environ['FIGS_OUT'], 
                         'methods_box8_2019_clusters_hist.pdf'), dpi=300)

# %% trusted=true

# %% trusted=true
# %matplotlib widget


plt.figure(figsize=(3,2))
crs = ccrs.NorthPolarStereo(central_longitude=-45., true_scale_latitude=70.)
ax = plt.axes(projection=crs)

plt.plot(*slices_b8_centre.geometry.exterior.xy, zorder=1, color='black', alpha=0.2, linewidth=1)


df_labelled_gpd = gpd.GeoDataFrame(df_labelled, geometry=gpd.points_from_xy(df_labelled.x, df_labelled.y), crs=3413)
df_labelled_gpd = df_labelled_gpd[df_labelled_gpd.within(slices_b8_centre.geometry)]
chosen = df_labelled_gpd.groupby('label').elev.median().sort_values(ascending=False)
chosen = chosen.index[0]
df_labelled_gpd.loc[:, 'chosen'] = np.where(df_labelled_gpd.label == chosen, 1, 0)
small_palette = [palette[i] for i in sorted(df_labelled_gpd.label.unique())]
sns.scatterplot(data=df_labelled_gpd, x='x', y='y', hue='label', 
               legend=False, alpha=0.4, palette=small_palette, zorder=100, 
                style='chosen', markers=['o', '^'])
picked_val = rlims.xytpd[(rlims.xytpd.box_id == 8) & (rlims.xytpd.slice_id == 50) & (rlims.xytpd.year == 2019)]
plt.plot(picked_val.x, picked_val.y, 's', zorder=100, mec='black', mfc='none', markersize=6)

# Also show the e_max value
e_max = pd.read_csv(os.path.join(os.environ['ANALYSIS_OUT'], 'rlim_annual_maxm', 'xytpd_emax.csv'))
e_max_here = e_max[(e_max.box_id == 8) & (e_max.slice_id == 50) & (e_max.year == 2019)]
plt.plot(e_max_here.x, e_max_here.y, 'P', color='black', markersize=6)

scalebar.scale_bar(ax, (0.1, 0.1), 20, linewidth=1)

plt.ylim(-2553321, -2543057)
plt.xlim(-117842, -85414)

plt.savefig(os.path.join(os.environ['FIGS_OUT'], 
                         'methods_box8_2019_slice50.pdf'), dpi=300)

# %% trusted=true
e_max

# %% trusted=true
chosen = df_labelled_gpd.groupby('label').elev.median().sort_values(ascending=False)

# %% trusted=true
chosen

# %% trusted=true
#### DEPRECATED 2021-09-21 - this version is based around zonal polynomials, we now use regional - see next cell.

## Plot seasonality/E_max lookup example
# This is hard-coded for box 8, slice 50, using 2019 data.
# I've had to copy-paste code from seasonality_for_max_rlim.py in order to make subplot (a).
"""
%matplotlib widget
plt.figure(figsize=(3,2.5))

ax = plt.subplot(2,1,1)

#### COPIED FROM ORIGINAL SCRIPT VVVVVVVVVVVVV ##############
from sklearn.preprocessing import StandardScaler
# Identify products which are clearly very noisy.
retain_ids = df.groupby('product_id').elev.mad()
retain_ids.name = 'product_id__mad'
retain_ids = retain_ids[retain_ids < 75] 

# Delete the noisy products.
df_culled = pd.merge(left=df, right=retain_ids, how='right', 
    left_on='product_id', right_index=True)

# Do scaling
outputs = []
for year in df_culled.year.unique():
    df_year = df_culled[df_culled.year == year]

    # do some filtering to get rid of very small values
    #df_year = df_year[df_year.elev > ]
    ts = pd.DatetimeIndex(df_year.date)
    df_year.loc[:,'doy'] = ts.dayofyear

    dd = df_year.filter(items=['elev'], axis='columns').to_numpy()
    if len(dd) > 0:
        scaler = StandardScaler(with_std=False).fit(dd)

        norm = scaler.transform(dd)    

        outputs.append(pd.Series(norm.squeeze(), index=df_year.doy))

all_data = pd.concat(outputs).to_frame()
#### END COPY #############################


# Plot the underlying data density
all_data.loc[:,'doy'] = all_data.index
all_data = all_data.rename({0:'elev'}, axis='columns')
sns.histplot(data=all_data, x='doy', y='elev', color='grey')  

# Lookup calculated coefficients and plot them
season_coefs = pd.read_csv(os.path.join(os.environ['ANALYSIS_OUT'], 'seasonality', 'box_polynomial_coefs.csv'))
doy_range = np.arange(182, 245)

coefs = season_coefs.loc[8]

ediff = (coefs.c4 * doy_range**3) + \
    (coefs.c3 * doy_range**2) +     \
    (coefs.c2 * doy_range) +        \
    coefs.c1
ediffs = pd.Series(ediff, index=doy_range)

ediffs.plot(color='tab:red', linewidth=2)
plt.ylim(-250, 250)
plt.ylabel(r'Norm. $\grave \upsilon$ (m)')
plt.xlabel('Day of year')

ax.annotate('a', fontsize=9, fontweight='bold', xy=(0.05, 0.95), xycoords='axes fraction',
           horizontalalignment='left', verticalalignment='top')



### Show an example look up.
ax= plt.subplot(2,1,2)

val = rlims.xytpd[(rlims.xytpd.box_id == 8) & (rlims.xytpd.year == 2019) & (rlims.xytpd.slice_id==50)]
val = val.iloc[0]
yi = ediffs.loc[val.date.dayofyear]
adj_ediffs = ediffs - yi + val.elev
adj_ediffs.plot(color='tab:red', linewidth=2, alpha=0.5)

plt.plot(val.date.dayofyear, val.elev, 'o', mec='black', mfc='none', markersize=7)

max_e = adj_ediffs.idxmax()
plt.plot(max_e, adj_ediffs.loc[max_e], 'o', mec='tab:red', mfc='none', markersize=7)

plt.ylim(1850, 1950)
plt.ylabel(r'$\grave \upsilon$ (m)')

plt.xlabel('Day of year 2019')
ax.annotate('b', fontsize=9, fontweight='bold', xy=(0.05,0.95), xycoords='axes fraction',
           horizontalalignment='left', verticalalignment='top')


sns.despine()
plt.tight_layout(h_pad=-0.4)

plt.savefig(os.path.join(os.environ['FIGS_OUT'], 
                         'methods_box8_2019_seasonality.pdf'))
"""

# %% trusted=true
### 2021-09-21: following switch from zones to regions for the polynomial fits, this Methods figure
# no longer needs the zone-specific fit - as the region-specific one is now provided in the temporal box plot.

# %matplotlib widget
plt.figure(figsize=(3,1.8))

### Show an example look up.
ax= plt.subplot(1,1,1)

season_coefs = pd.read_csv(os.path.join(os.environ['ANALYSIS_OUT'], 'seasonality', 'region_polynomial_d2_coefs.csv'),
                          index_col='region')
doy_range = np.arange(182, 245)

coefs = season_coefs.loc['SW']

ediff = (coefs.c3 * doy_range**2) +     \
    (coefs.c2 * doy_range) +        \
    coefs.c1
ediffs = pd.Series(ediff, index=doy_range)

val = rlims.xytpd[(rlims.xytpd.box_id == 8) & (rlims.xytpd.year == 2019) & (rlims.xytpd.slice_id==50)]
val = val.iloc[0]
yi = ediffs.loc[val.date.dayofyear]
adj_ediffs = ediffs - yi + val.elev
adj_ediffs.plot(color='tab:red', linewidth=2, alpha=0.5)

plt.plot(val.date.dayofyear, val.elev, 'o', mec='black', mfc='none', markersize=7)

max_e = adj_ediffs.idxmax()
plt.plot(max_e, adj_ediffs.loc[max_e], 'o', mec='tab:red', mfc='none', markersize=7)

plt.ylim(1850, 1950)
plt.ylabel(r'$\grave \upsilon$ (m)')

plt.xlabel('Day of year 2019')
ax.annotate('e', fontsize=9, fontweight='bold', xy=(0.05,0.95), xycoords='axes fraction',
           horizontalalignment='left', verticalalignment='top')


sns.despine()
plt.tight_layout(h_pad=-0.4)

plt.savefig(os.path.join(os.environ['FIGS_OUT'], 
                         'methods_box8_2019_seasonality.pdf'))
