"""
Create raster masks of the regions specified by load_env regions_boxes for the 
version of MAR specified in load_env.
"""

from load_env import *

gis = gisloader()

def make_rasters(rcm_ds, dest):
    region_msk_store = []
    for region in gis.regions_boxes:
        bmin, bmax = gis.regions_boxes[region]
        boxes_here = gis.boxes[(gis.boxes.FID >= bmin) & (gis.boxes.FID <= bmax)]
        # The function handles the translation to MAR CRS for us.
        image = polygon_to_mar_mask(rcm_ds, polygons=boxes_here)
        region_msk_store.append(image)

    region_msks = xr.DataArray(region_msk_store, dims=('region', 'y', 'x'), 
        coords={'region':list(gis.regions_boxes.keys()), 
        'y':rcm_ds.y, 'x':rcm_ds.x})

    region_msks.to_netcdf(dest)

# Make MAR rasters
try:
    make_rasters(gis.mar_ds, gis.fn_regions_as_mar_masks)
except PermissionError as e:
    print('MAR: file already exists!')
    print(e)

# Make RACMO rasters
try:
    make_rasters(gis.racmo_ds, gis.fn_regions_as_racmo_masks)
except PermissionError as e:
    print('RACMO: file already exists!')
    print(e)