"""
An example script for filtering labels to remove the ones that we don't want.

This is executed within the FilterFeaturesTask of ppj_runoff_limit.py.

Note that your version of this script has to be located in $OUTPUT_DIR/<job_id>/,
if it remains in $SCRIPTS then it will not be found!

"""

# Remove regions whose skeletal length is too short
# 50 px = 1.5 km
# 67 px = just over 2 km
# 34 px = 1.02 km
self._queue_labels(skel_regions[(skel_regions['area'] < 34)])

# Remove any features above what we consider to be maxium plausible elev limit
self._queue_labels(regions[(skel_regions['mean_dem'] >= 2100)])

# Remove any near-perfectly 'circular' features - these are likely to be lakes
# 0 eccentricity == perfect circle
self._queue_labels(regions[(regions['eccentricity'] < 0.2) & (regions['area'] < km2_to_px(2.))])

# Remove regions with an overall small area 
# (as the skeleton criterion isn't enough, as this can branch in multiple directions)
self._queue_labels(regions[(regions['area'] < km2_to_px(0.5))])

# Remove regions which definitely don't have water reflectance included
self._queue_labels(regions[regions['min_B4'] > 0.2])

# Generate an 'interim' filtered features set
skel_regions_filt = skel_regions.drop(labels=self._labels_to_drop, axis='index')

# Use this 'interim' set to find features in the highest DEM
# quantile of the dataset, then retain only the very largest drainage
# features at this elevation.
perc = skel_regions_filt.max_dem.quantile(.9)
self._queue_labels(regions[(regions['area'] < km2_to_px(1.5)) & (skel_regions['max_dem'] >= perc)])