from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import Column, Integer, String
from sqlalchemy.orm import sessionmaker

import geopandas as gpd
import pandas as pd
import os
import argparse
import datetime as dt

parser = argparse.ArgumentParser(description='Insert filtered runoff limits into db')
parser.add_argument('job_id', type=str, 
	help='str, job ID, as stored in $OUTPUT_DIR')
parser.add_argument('-force_add', dest='force_add', default=False, action='store_true',
	help='if provided, force adding job to DB even if this job ID already exists in DB. This will result in .job.n being generated.')
args = parser.parse_args()


conn = create_engine("postgresql://tedstona@localhost/rlims") 
Session = sessionmaker(bind=conn)
session = Session()

sql = "SELECT job_name from jobs WHERE job_name = '{name}'".format(name=args.job_id)
res = pd.read_sql(sql, conn)
insert_job_id = args.job_id
if len(res) > 0:
	if args.force_add:
		insert_job_id = args.job_id + '.' + dt.datetime.now().strftime('%y%m%d%H%M%S')
		print('Forcing addition, with Job ID: {j}'.format(j=insert_job_id))
	else:
		raise ValueError('This job is already saved in database.')


csv_str = '_RLIM_filtered_MM.csv' 
fn = os.path.join(os.environ['OUTPUT_DIR'], args.job_id, args.job_id+csv_str)

# Load and clean rlims for the job
data = pd.read_csv(fn)
to_drop = ['Unnamed: 0', 'Unnamed: 0.1', 'row_rot', 'i', 'dist', 'rlim', 'geometry']
data = data.drop(columns=to_drop)
data = data.rename({'PRODUCT_ID':'product_id'}, axis=1)

# Prepare the job ID for insertion
Base = declarative_base()
class Jobs(Base):
    __tablename__ = 'jobs'
    jid = Column(Integer, primary_key=True)
    job_name = Column(String)

job = Jobs(job_name=insert_job_id)
session.add(job)
session.commit()
    

# Update the dataframe with the job_id integer
data['jid'] = job.jid

# Save the dataframe to the db
data.to_sql('rlims', conn, if_exists='append', index=False)

# Use native PostGIS to set the geometry
sql = """UPDATE rlims
SET geometry = ST_GeomFromText('POINT(' || x || ' ' || y || ')',3413) WHERE geometry IS NULL;"""
session.execute(sql)
session.commit()

session.close()

#res = gpd.read_postgis("select rlims.elev, rlims.geometry from rlims join boxes on st_contains(boxes.geom, rlims.geometry) where boxes.gid=1", conn, geom_col='geometry')  
#si = res.sindex