# Help sources:
# https://www.kevfoo.com/2012/01/Importing-CSV-to-PostGIS/
# https://gist.github.com/gwangjinkim/f13bf596fefa7db7d31c22efd1627c7a



export LC_ALL="en_US.UTF-8"                                                                                       
export LC_CTYPE="en_US.UTF-8"

# Initialising database - once only
initdb -D rlims  
pg_ctl -D rlims -l logfile start 
createuser --encrypted --pwprompt nonsuper     pwd 123456                                                                   
createdb --owner=nonsuper                                                                                        
createdb --owner=nonsuper rlims    


CREATE TABLE rlims (
	gid serial NOT NULL,
	x double precision,
	y double precision,
	row_northup integer,
	col_northup integer,
	tot_cldpx_above_roff double precision,
	tot_clouds_row double precision,
	nearest_cld double precision,
	px_to_cld_up double precision,
	px_to_cld_down double precision,
	px_to_pr double precision,
	elev double precision,
	date date,
	PRODUCT_ID varchar(50),
	geometry geometry,
	dist double precision,

	jid int NOT NULL,
	CONSTRAINT fk_jid FOREIGN KEY (jid) REFERENCES jobs (jid) ON DELETE CASCADE,

	CONSTRAINT enforce_dims_geometry CHECK (st_ndims(geometry) = 2),
	CONSTRAINT enforce_geotype_geometry CHECK (geometrytype(geometry) = 'POINT'::text OR geometry IS NULL),
	CONSTRAINT enforce_srid_geometry CHECK (st_srid(geometry) = 3413)
);

CREATE INDEX rlims_geometry_gist
ON rlims
USING gist(geometry);

CREATE TABLE jobs (
	jid serial NOT NULL,
	job_name varchar(50),
	PRIMARY KEY (jid)
);

# Index the foreign key too
CREATE INDEX rlims_jid_ix ON rlims (jid);


# Can use the psql \copy command - but I'm using a Python script instead
# https://courses.cs.washington.edu/courses/csep544/11au/resources/postgresql-instructions.html
\copy rlims(x,y,row_northup,col_northup,tot_cldpx_above_roff,tot_clouds_row,nearest_cld,px_to_cld_up,px_to_cld_down,px_to_pr,elev,date,PRODUCT_ID,dist) FROM '/flash/tedstona/LC8_SW.20200604-151900.job/LC8_SW.20200604-151900.job_RLIM_filtered_MM_dropcols.csv' DELIMITERS ',' CSV HEADER;
UPDATE rlims
SET geometry = ST_GeomFromText('POINT(' || x || ' ' || y || ')',3413);



# Load shapefile: https://postgis.net/docs/using_postgis_dbmanagement.html#shp2pgsql_usage
shp2pgsql -c -D -I -s 3413 /home/geoscience/nobackup_cassandra/L0data/100k_boxes/boxes.shp > /home/geoscience/nobackup_cassandra/L0data/100k_boxes/boxes.sql
psql
\i /home/geoscience/nobackup_cassandra/L0data/100k_boxes/boxes.sql;


# Example of a a spatial join
select rlims.elev from rlims join boxes on st_contains(boxes.geom, rlims.geometry) where boxes.gid=1; 
# More info on spatial joins: http://postgis.net/workshops/postgis-intro/joins_advanced.html


#Help: \?
# List tables: \dt