import numpy as np
import os
import subprocess

from eolearn.core import EOTask
from eolearn.io.local_io import ExportToTiff, ImportFromTiff
from eolearn.core.constants import FeatureType


class ExportAndWarpTask(EOTask):

	def __init__(self, feature, folder, epsg, resolution, 
			dtype=np.int16, no_data_value=np.nan,
			compress=True, cleanup=False):
		
		self.feature = self._parse_features(feature)
		self.folder = folder
		self.dtype = dtype
		self.no_data_value = no_data_value

		self.epsg = epsg
		self.res = resolution

		self.compress = compress
		self.cleanup = cleanup
		return


	def execute(self, eopatch, filename):

		feature_type, feature_name = next(self.feature())

		# Create tiff
		t = ExportToTiff((feature_type, feature_name.upper()), 
				folder=self.folder, image_dtype=self.dtype,
				no_data_value=self.no_data_value, compress='lzw') 

		t.execute(eopatch, filename=filename)
		
		# Compression ooptions
		if self.compress:
			compress_opt = '-co "COMPRESS=LZW"'
		else:
			compress_opt = ''

		# Sort filenames
		fn_old = os.path.join(self.folder, filename)
		fn_new = ('{fn}_EPSG{epsg}.tif').format(fn=fn_old[:-4], epsg=self.epsg)

		# the -tap option is essential
		cmd = ('gdalwarp {a} -tap -overwrite -of GTiff -tr {b} {b} '
			'-t_srs EPSG:{c} {d} {e}').format(
				a=compress_opt, b=self.res, c=self.epsg, d=fn_old, e=fn_new)
		subprocess.check_output(cmd, shell=True)

		if self.cleanup:
			cmd = ('rm {old}').format(old=fn_old)
			subprocess.check_output(cmd, shell=True)

		return eopatch