import numpy as np
from scipy import ndimage as nd

from skimage.filters import gabor_kernel



def normalise(image):
	"""
	Normalise image.

	This may need further work.
	"""

	return (image - image.mean()) / image.std()




def run(image, kernel):
	"""
	Run gabor filter over specified direction

	:param image: float image
	:type image: np.array 
	:param kernel: Operating kernel
	:type kernel: np.array

	:return: filtered image
	:type: np.array

	!TODO: consider converting to cython function
	"""

	return np.sqrt(
		nd.convolve(image, np.real(kernel), mode='wrap') ** 2 +
		nd.convolve(image, np.imag(kernel), mode='wrap') ** 2
	)




def design_kernels_evenspaced(ndirs, freq, **kwargs):
	"""
	Design filter kernels for across ndir equally-space directions of a circle

	:param ndirs: number of directions to design for
	:type ndirs: int
	:param freq: frequency of gabor filter
	:type freq: float

	:return: list of kernels, each an np.array
	:rtype: list
	"""

	kernels = []
	for direc in range(ndirs):
		theta = (180 / ndirs * direc) * (np.pi / 180)
		kernel = gabor_kernel(freq, theta=theta, **kwargs)
		kernels.append(kernel)

	return kernels
